@@ -3,22 +3,26 @@
  *
  *	Copyright (C) 1993 by the Regents of the University of California
  *
- *      This program is free software; you can redistribute it and/or modify
+ *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
+ *      the Free Software Foundation, either version 3 of the License, or
  *      (at your option) any later version.
- *  
+ *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
- *  
+ *
  *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *      along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
+#ifdef HAVE_WX
+#define fgets wx_fgets
+extern int check_wx_stop(int force_yield);
+#endif
+
 #define WANT_EVAL_REGS 1
 #include "logo.h"
 #include "globals.h"
@@ -41,7 +45,11 @@
 #endif
 
 #ifdef HAVE_TERMIO_H
+#ifdef HAVE_WX
+#include <termios.h>
+#else
 #include <termio.h>
+#endif
 #else
 #ifdef HAVE_SGTTY_H
 #include <sgtty.h>
@@ -90,7 +98,7 @@ NODE *loutput(NODE *arg) {
 }
 
 NODE *lstop(NODE *args) {
-    if (NOT_THROWING)
+  if (NOT_THROWING) {
 	if (ufun == NIL)
 	    err_logo(AT_TOPLEVEL, theName(Name_stop));
 	else if (val_status & OUTPUT_TAIL 
@@ -101,6 +109,7 @@ NODE *lstop(NODE *args) {
 	    stopping_flag = STOP;
 	    output_unode = current_unode;
 	}
+  }
     return(UNBOUND);
 }
 
@@ -299,7 +308,7 @@ NODE *lforever(NODE *args) {
 NODE *ltest(NODE *args) {
     int arg = torf_arg(args);
 
-    if (tailcall != 0) return UNBOUND;
+    if (ufun != NIL && tailcall != 0) return UNBOUND;
     if (NOT_THROWING) {
 	dont_fix_ift = arg+1;
     }
@@ -324,13 +333,22 @@ NODE *liffalse(NODE *args) {
 	return(NIL);
 }
 
+
 void prepare_to_exit(BOOLEAN okay) {
+
+#ifdef HAVE_WX
+	extern void doClose();
+	doClose();
+	wxLogoExit (0);
+#endif
+
 #ifdef mac
     if (okay) {
 	console_options.pause_atexit = 0;
 	exit(0);
     }
 #endif
+
 #ifndef WIN32 /* sowings */
 #ifdef ibm
     ltextscreen(NIL);
@@ -367,13 +385,22 @@ NODE *lbye(NODE *args) {
 #ifdef __RZTC__
     printf("\n");
 #endif
+    
+#ifdef HAVE_WX
+    wx_leave_mainloop++;
+    return UNBOUND;
+#else    
     exit(0);
     return UNBOUND;	/* not reached, but makes compiler happy */
+#endif
 }
 
 NODE *lwait(NODE *args) {
     NODE *num;
     unsigned int n;
+#if defined(unix) && HAVE_USLEEP
+	unsigned int seconds, microseconds;
+#endif
 #ifdef mac
     long target;
     extern void ProcessEvent(void);
@@ -381,19 +408,44 @@ NODE *lwait(NODE *args) {
 
     num = pos_int_arg(args);
     if (NOT_THROWING) {
+      /*#ifdef HAVE_WX
+      n = (unsigned int)getint(num) * 10; // milliseconds
+      wxLogoSleep(n);
+      return(UNBOUND);
+      #endif*/
+#ifndef HAVE_WX
 #ifdef WIN32
-	win32_update_text();
+      win32_update_text();
 #else
-	fflush(stdout); /* csls v. 1 p. 7 */
+      fflush(stdout); /* csls v. 1 p. 7 */
 #endif
+#else
+      //doesn't seem to work in WX. now done in wxLogoSleep
+      //fflush(stdout); /* csls v. 1 p. 7 */
+#endif
+
 #if defined(__RZTC__)
 	zflush();
 #endif
+	fix_turtle_shownness();
+
+#ifdef HAVE_WX
+	n = (unsigned int)getint(num) * 100 / 6; // milliseconds
+	wxLogoSleep(n);
+	//check_throwing;
+	return(UNBOUND); 
+#endif
+
 	if (getint(num) > 0) {
 #ifdef unix
 #ifdef HAVE_USLEEP
-	    n = (unsigned int)getint(num) * 16667;
-	    usleep(n);
+	    n = (unsigned int)getint(num);
+		
+		if (seconds = n / 60)
+			sleep(seconds);
+			
+		if (microseconds = (n % 60) * 16667)
+			usleep(microseconds);
 #else
 	    n = (unsigned int)getint(num) / 60;
 	    sleep(n);
@@ -428,7 +480,7 @@ NODE *lwait(NODE *args) {
 }
 
 NODE *lshell(NODE *args) {
-#ifdef mac
+#if defined(mac)
     printf("%s\n", message_texts[NOSHELL_MAC]);
     return(UNBOUND);
 #else    
@@ -436,6 +488,8 @@ NODE *lshell(NODE *args) {
     NODE *arg;
     char doscmd[200];
 /*  union REGS r;     */
+    char *old_stringptr = print_stringptr;
+    int old_stringlen = print_stringlen;
 
     arg = car(args);
     while (!is_list(arg) && NOT_THROWING) {
@@ -458,6 +512,8 @@ NODE *lshell(NODE *args) {
 	    err_logo(FILE_ERROR,
 	      make_static_strnode
 		 ("Could not open shell (probably due to low memory)"));
+	print_stringptr = old_stringptr;
+	print_stringlen = old_stringlen;
     }
 /*
     r.h.ah = 0x3;
@@ -482,13 +538,21 @@ NODE *lshell(NODE *args) {
     NODE *head = NIL, *tail = NIL, *this;
     BOOLEAN wordmode = FALSE;
     int len;
+    char *old_stringptr = print_stringptr;
+    int old_stringlen = print_stringlen;
 
     if (cdr(args) != NIL) wordmode = TRUE;
     print_stringptr = cmdbuf;
     print_stringlen = 300;
     ndprintf((FILE *)NULL,"%p\n",car(args));
     *print_stringptr = '\0';
+#ifdef __WXMSW__
+	strm = _popen(cmdbuf,"r");
+#else
     strm = popen(cmdbuf,"r");
+#endif
+    print_stringptr = old_stringptr;
+    print_stringlen = old_stringlen;
     fgets(cmdbuf,300,strm);
     while (!feof(strm)) {
 	len = (int)strlen(cmdbuf);
@@ -507,7 +571,11 @@ NODE *lshell(NODE *args) {
 	}
 	fgets(cmdbuf,300,strm);
     }
+#ifdef __WXMSW__
+	_pclose(strm);
+#else
     pclose(strm);
+#endif
     return(head);
 #endif
 #endif
