@@ -3,19 +3,18 @@
  *
  *	Copyright (C) 1993 by the Regents of the University of California
  *
- *      This program is free software; you can redistribute it and/or modify
+ *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
+ *      the Free Software Foundation, either version 3 of the License, or
  *      (at your option) any later version.
- *  
+ *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
- *  
+ *
  *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *      along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -24,7 +23,11 @@
 #include "globals.h"
 
 #ifdef HAVE_TERMIO_H
+#ifdef HAVE_WX
+#include <termios.h>
+#else
 #include <termio.h>
+#endif
 #else
 #ifdef HAVE_SGTTY_H
 #include <sgtty.h>
@@ -54,6 +57,8 @@ void err_print(char *buffer) {
     int errtype;
     NODE *errargs, *oldfullp;
     FILE *fp;
+    char *old_stringptr = print_stringptr;
+    int old_stringlen = print_stringlen;
 
     if (err_mesg == NIL) return;
 
@@ -89,9 +94,11 @@ void err_print(char *buffer) {
     err_mesg = NIL;
     if (buffer == NULL)
 	new_line(fp);
-    else
+    else {
 	*print_stringptr = '\0';
-
+	print_stringptr = old_stringptr;
+	print_stringlen = old_stringlen;
+    }
     setvalnode__caseobj(Fullprintp, oldfullp);
     stopping_flag = save_flag;
 }
@@ -151,9 +158,10 @@ NODE *err_logo(ERR_TYPES error_type, NODE *error_desc) {
 	case IS_PRIM:
 	case AT_TOPLEVEL:
 	case ERR_MACRO:
+	case RUNNABLE_ARG:
+	case TOO_MUCH:
 	case DEEPEND:
 	case BAD_DEFAULT:
-	case TOO_MUCH:
 	case CANT_OPEN_ERROR:
 	case ALREADY_OPEN_ERROR:
 	case NOT_OPEN_ERROR:
@@ -241,12 +249,14 @@ NODE *lerror(NODE *args) {
     return(val);
 }
 
+/*
 #ifndef HAVE_MEMCPY
 void memcpy(char *to, char *from, size_t len) {
     while (--len >= 0)
 	*to++ = *from++;
 }
 #endif
+*/
 
 NODE *lpause(NODE *args) {
     NODE *elist = NIL, *val = UNBOUND, *uname = NIL;
@@ -256,9 +266,10 @@ NODE *lpause(NODE *args) {
 #endif
 
     if (err_mesg != NIL) err_print(NULL);
- /* if (ufun != NIL) */ {
+    ndprintf(stdout, "%t\n", message_texts[PAUS_ING]);
+    if (inside_evaluator) {
 	uname = ufun;
-	ndprintf(stdout, "%t\n", message_texts[PAUS_ING]);
+	ufun = NIL;
 #ifndef TIOCSTI
 	memcpy((char *)(&sav_iblk), (char *)(&iblk_buf), sizeof(jmp_buf));
 #endif
@@ -311,7 +322,15 @@ NODE *lpause(NODE *args) {
 /*  } else {
 	stopping_flag = THROWING;
 	throw_node = theName(Name_toplevel);
- */ }
+ */
+    } else {
+	ndprintf(stdout, "? ");
+#ifdef HAVE_WX
+	flushFile(stdout);
+#else
+	fflush(stdout);
+#endif
+}
     return(val);
 }
 
