@@ -3,25 +3,26 @@
  *
  *	Copyright (C) 1993 by the Regents of the University of California
  *
- *      This program is free software; you can redistribute it and/or modify
+ *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
+ *      the Free Software Foundation, either version 3 of the License, or
  *      (at your option) any later version.
- *  
+ *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
- *  
+ *
  *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *      along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
 /* main.c */
 #ifdef HAVE_WX
 extern int start(int, char **);
+extern int wx_leave_mainloop;
+extern int check_wx_stop(int force_yield);
 #endif
 extern NODE **bottom_stack; /*GC*/
 extern NODE *current_line, *exec_list;
@@ -29,6 +30,7 @@ extern int main(int, char *[]);
 extern void unblock_input(void);
 extern NODE **bottom_stack;
 extern void delayed_int(void);
+extern NODE *command_line;
 
 #if defined(SIG_TAKES_ARG)
 RETSIGTYPE logo_stop(int);
@@ -51,6 +53,7 @@ extern jmp_buf iblk_buf;
 extern char *strnzcpy(char *, char *, int);
 extern char *word_strnzcpy(char *, NODE *, int);
 extern char *noparity_strnzcpy(char *, char *, int);
+extern char *backslashed_strnzcpy(char *, char *, int);
 extern char *mend_strnzcpy(char *, char *, int);
 extern char *mend_nosemi(char *, char *, int);
 extern char *low_strnzcpy(char *, char *, int);
@@ -101,7 +104,7 @@ extern NODE *oldyoungs;
 extern BOOLEAN inside_gc, int_during_gc;
 extern BOOLEAN addseg(void);
 extern NODETYPES nodetype(NODE *);
-extern void check_valid_oldyoung(NODE *old, NODE *new);
+extern void check_valid_oldyoung(NODE *old, NODE *n);
 extern void setobject(NODE *, NODE *);
 extern void setcar(NODE *, NODE *);
 extern void setcdr(NODE *, NODE *);
@@ -125,6 +128,10 @@ extern NODE *parser(NODE *, BOOLEAN);
 extern NODE *lparse(NODE *);
 extern NODE *runparse(NODE *);
 extern NODE *lrunparse(NODE *);
+#if 0
+extern char *cmdHistory[];
+extern char **hist_inptr, **hist_outptr;
+#endif
 
 /* math.c */
 extern int numberp(NODE *);
@@ -200,14 +207,18 @@ extern NODE *Redefp, *Caseignoredp, *Erract, *Printdepthlimit;
 extern NODE *Printwidthlimit, *Pause, *LoadNoisily, *AllowGetSet;
 extern NODE *UnburyOnEdit, *Make, *Listvalue, *Dotsvalue;
 extern NODE *Unbound, *Not_Enough_Node, *Buttonact, *LogoVersion;
-extern NODE *Minus_Sign, *Minus_Tight, *Startup, *Query;
-extern NODE *UseAlternateNames, *LogoLogo, *LogoPlatform;
-extern NODE *Null_Word;
+extern NODE *Minus_Sign, *Minus_Tight, *Startup, *Startuplg, *Query;
+extern NODE *UseAlternateNames, *LogoLogo, *LogoPlatform, *Keyact;
+extern NODE *CommandLine, *Null_Word;
 extern void init(void);
 extern struct wdtrans translations[];
 extern char *LogoPlatformName;
 
 /* wrksp.c */
+#ifdef HAVE_WX
+extern char* wx_fgets(char* s, int n, FILE* stream);
+#endif
+
 extern char *editor, *editorname, *tempdir;
 extern int to_pending;
 extern NODE *ltext(NODE *);
@@ -286,6 +297,9 @@ extern int not_local(NODE *, NODE *);
 extern int num_saved_nodes;
 extern struct registers regs;
 extern NODE *Regs_Node;
+extern int inside_evaluator;
+extern NODE *evaluator(NODE *list, enum labels where);
+extern NODE *eval_buttonact;
 
 /*
  #ifdef OBJECTS
@@ -304,7 +318,7 @@ extern NODE *llist(NODE *);
 extern NODE *lemptyp(NODE *);
 extern NODE *lascii(NODE *);
 extern NODE *lrawascii(NODE *);
-extern NODE *lbackslashedp(NODE *);
+extern NODE *lvbarredp(NODE *);
 extern NODE *lchar(NODE *);
 extern NODE *lcount(NODE *);
 extern NODE *lfput(NODE *);
@@ -332,6 +346,9 @@ extern NODE *l_setfirst(NODE *);
 extern NODE *l_setbf(NODE *);
 
 /* files.c */
+#ifdef HAVE_WX
+extern int wxKeyp();
+#endif
 extern NODE *file_list;
 extern NODE *reader_name, *writer_name, *file_prefix;
 extern NODE *lseteditor(NODE *);
@@ -369,6 +386,9 @@ extern NODE *lreadpos(NODE *);
 extern NODE *lsetreadpos(NODE *);
 extern NODE *lwritepos(NODE *);
 extern NODE *lsetwritepos(NODE *);
+extern int readchar_lookahead_buf;
+extern int need_save;
+extern NODE *save_name;
 
 /* coms.c */
 extern NODE *make_cont(enum labels, NODE *);
@@ -397,9 +417,12 @@ extern void prepare_to_exit(BOOLEAN);
 extern NODE *lbye(NODE *);
 extern NODE *lwait(NODE *);
 extern NODE *lshell(NODE *);
+NODE *runnable_arg(NODE *);
 
 /* term.c */
+#ifndef HAVE_WX
 extern int x_coord, y_coord, x_max, y_max;
+#endif
 extern int interactive;
 extern void term_init(void);
 extern void charmode_on(void);
@@ -419,8 +442,24 @@ extern void untreeify_proc(NODE *);
 extern void make_tree_from_body(NODE *);
 extern void make_tree(NODE *);
 extern NODE *tree_dk_how;
+extern void check_library(NODE *);
 
 /* graphics.c */
+#ifdef HAVE_WX
+#ifndef IN_SPLITSCREEN
+#define REMOVE_DEFINES 1
+#define	IN_SPLITSCREEN		4
+#define	IN_GRAPHICS_MODE	5
+#endif
+int wxGetInfo(int);
+#define in_splitscreen wxGetInfo(IN_SPLITSCREEN)
+#define in_graphics_mode wxGetInfo(IN_GRAPHICS_MODE)
+
+#ifdef REMOVE_DEFINES
+#undef IN_SPLITSCREEN
+#undef IN_GRAPHICS_MODE
+#endif
+#endif
 extern mode_type current_mode;
 extern FLONUM turtle_x, turtle_y, turtle_heading, x_scale, y_scale;
 extern BOOLEAN turtle_shown;
@@ -452,6 +491,9 @@ extern NODE *lwrap(NODE *);
 extern NODE *lfence(NODE *);
 extern NODE *lwindow(NODE *);
 extern NODE *lfill(NODE *);
+#ifdef HAVE_WX
+extern NODE *lfilled(NODE *);
+#endif
 extern NODE *llabel(NODE *);
 extern NODE *ltextscreen(NODE *);
 extern NODE *lsplitscreen(NODE *);
@@ -475,6 +517,7 @@ extern NODE *lsetpensize(NODE *);
 extern NODE *lsetpenpattern(NODE *);
 extern NODE *lsetscrunch(NODE *);
 extern NODE *lmousepos(NODE *);
+extern NODE *lclickpos(NODE *);
 extern NODE *lbuttonp(NODE *);
 extern NODE *lbutton(NODE *);
 extern NODE *ltone(NODE *);
@@ -487,6 +530,11 @@ extern NODE *lepspict(NODE *);
 extern void redraw_graphics(void);
 extern NODE *lscreenmode(NODE *);
 extern NODE *lturtlemode(NODE *);
+extern void fix_turtle_shownness(void);
+extern enum s_md {SCREEN_TEXT, SCREEN_SPLIT, SCREEN_FULL} screen_mode;
+#ifdef HAVE_WX
+extern NODE *lprintpict(NODE *), *lprinttext(NODE *);
+#endif
 
 #ifdef mac
 
@@ -514,6 +562,17 @@ extern WindowPtr graphics_window, listener_window;
 #endif
 #endif
 
+#ifdef HAVE_WX
+extern NODE *lfont(NODE *);
+extern NODE *lsetlabelheight(NODE *);
+extern NODE *llabelsize(NODE *);
+extern NODE *lsetfont(NODE *);
+extern NODE *lsettextsize(NODE *);
+extern NODE *ltextsize(NODE *);
+extern NODE *set_text_color(NODE *);
+#endif
+
+
 #ifdef __RZTC__   /* ztcterm.c */
 extern BOOLEAN in_graphics_mode, in_splitscreen;
 extern int ibm_screen_bottom;
@@ -545,8 +604,37 @@ extern NODE *set_text_color(NODE *);
 #ifdef HAVE_WX
 extern void init_wx(int, char**);
 extern void printToScreen(char c, FILE * stream);
+extern void flushFile(FILE * stream);
 extern char getFromWX();
 extern char getFromWX_2(FILE * f);
+extern void wxLogoExit(int code);
+extern void wxLogoSleep(unsigned int milli);
+extern int wxUnget_c(int c, FILE * f);
+extern void wxlPrintPict(void);
+extern void wxlPrintPreviewPict(void);
+extern void wxlPrintText(void);
+extern void wxlPrintPreviewText(void);
+extern NODE*IncreaseFont(void *);
+extern NODE*DecreaseFont(void *);
+extern int turtlePosition_x;
+extern int turtlePosition_y;
+#define SCREEN_WIDTH		1
+#define SCREEN_HEIGHT		2
+#define	BACK_GROUND			3
+#define	IN_SPLITSCREEN		4
+#define	IN_GRAPHICS_MODE	5
+#define	X_COORD				6
+#define	Y_COORD				7
+#define X_MAX				8
+#define Y_MAX				9
+#define EDIT_STATE   	    10
+
+// used for the text editor flag
+#define NO_INFO 0
+#define DO_LOAD	1
+#define NO_LOAD 2
+#else
+extern int in_graphics_mode, in_splitscreen, cur_len, read_index;
 #endif
 
 #ifdef x_window
@@ -567,7 +655,7 @@ extern int clearing_screen;
 extern void WinDebug(char *);
 #endif
 
-extern int in_graphics_mode, in_splitscreen, cur_len, read_index;
+
 extern char *read_line, buffered_char;
 extern int char_mode;
 extern int line_avail, char_avail;
