@@ -3,19 +3,18 @@
  *
  *	Copyright (C) 1993 by the Regents of the University of California
  *
- *      This program is free software; you can redistribute it and/or modify
+ *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
+ *      the Free Software Foundation, either version 3 of the License, or
  *      (at your option) any later version.
- *  
+ *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
- *  
+ *
  *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *      along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -23,12 +22,21 @@
 #include <windows.h>
 #endif
 
+//#ifdef HAVE_UNISTD_H
+#ifndef WIN32
+#include <unistd.h>
+#endif
+
 #define WANT_EVAL_REGS 1
 #include "logo.h"
 #include "globals.h"
 #include <string.h>
 #include <time.h>
 
+#ifdef HAVE_WX
+void getExecutableDir(char * path, int maxlen);
+#endif
+
 typedef struct priminfo {
     char *name;
     short minargs;
@@ -41,8 +49,13 @@ typedef struct priminfo {
 NODE *Right_Paren, *Left_Paren, *Redefp, *Caseignoredp, *Erract, *Printdepthlimit,
      *Printwidthlimit, *Pause, *LoadNoisily, *AllowGetSet,
      *UnburyOnEdit, *Fullprintp, *Make, *Listvalue, *Dotsvalue,
-     *Unbound, *Not_Enough_Node,
-     *Minus_Sign, *Minus_Tight, *Startup, *Query, *UseAlternateNames;
+     *Unbound, *Not_Enough_Node, *Buttonact, *LogoVersion, *LogoPlatform,
+     *LogoLogo, *CommandLine, *Keyact,
+#ifdef OBJECTS
+     *askexist,
+#endif
+     *Minus_Sign, *Minus_Tight, *Startup, *Startuplg,
+     *Query, *UseAlternateNames;
 NODE *Null_Word = NIL;
 
 PRIMTYPE prims[] = {
@@ -62,6 +75,9 @@ PRIMTYPE prims[] = {
     {"<", 2, 2, 2, PREFIX_PRIORITY + 1, llessp},
     {"=", 2, 2, 2, PREFIX_PRIORITY + 1, lequalp},
     {">", 2, 2, 2, PREFIX_PRIORITY + 1, lgreaterp},
+    {"<=", 2, 2, 2, PREFIX_PRIORITY + 1, llessequalp},
+    {"<>", 2, 2, 2, PREFIX_PRIORITY + 1, lnotequalp},
+    {">=", 2, 2, 2, PREFIX_PRIORITY + 1, lgreaterequalp},
     {"?", 0, 0, 1, PREFIX_PRIORITY, lqm},
     {"allopen", 0, 0, 0, PREFIX_PRIORITY, lallopen},
     {"and", 0, 2, -1, PREFIX_PRIORITY, land},
@@ -75,10 +91,11 @@ PRIMTYPE prims[] = {
     {"array?", 1, 1, 1, PREFIX_PRIORITY, larrayp},
     {"ascii", 1, 1, 1, PREFIX_PRIORITY, lascii},
     {"ashift", 2, 2, 2, PREFIX_PRIORITY, lashift},
+#ifdef OBJECTS
+    {"ask", 2, 2, 2, MACRO_PRIORITY, lask},
+#endif
     {"back", 1, 1, 1, PREFIX_PRIORITY, lback},
     {"background", 0, 0, 0, PREFIX_PRIORITY, lbackground},
-    {"backslashedp", 1, 1, 1, PREFIX_PRIORITY, lbackslashedp},
-    {"backslashed?", 1, 1, 1, PREFIX_PRIORITY, lbackslashedp},
     {"beforep", 2, 2, 2, PREFIX_PRIORITY, lbeforep},
     {"before?", 2, 2, 2, PREFIX_PRIORITY, lbeforep},
     {"bf", 1, 1, 1, PREFIX_PRIORITY, lbutfirst},
@@ -97,6 +114,7 @@ PRIMTYPE prims[] = {
     {"butfirst", 1, 1, 1, PREFIX_PRIORITY, lbutfirst},
     {"butfirsts", 1, 1, 1, PREFIX_PRIORITY, lbfs},
     {"butlast", 1, 1, 1, PREFIX_PRIORITY, lbutlast},
+    {"button", 0, 0, 0, PREFIX_PRIORITY, lbutton},
     {"buttonp", 0, 0, 0, PREFIX_PRIORITY, lbuttonp},
     {"button?", 0, 0, 0, PREFIX_PRIORITY, lbuttonp},
     {"bye", 0, 0, 0, PREFIX_PRIORITY, lbye},
@@ -105,6 +123,9 @@ PRIMTYPE prims[] = {
     {"clean", 0, 0, 0, PREFIX_PRIORITY, lclean},
     {"clearscreen", 0, 0, 0, PREFIX_PRIORITY, lclearscreen},
     {"cleartext", 0, 0, 0, PREFIX_PRIORITY, lcleartext},
+#ifdef HAVE_WX
+    {"clickpos", 0, 0, 0, PREFIX_PRIORITY, lclickpos},
+#endif
     {"close", 1, 1, 1, PREFIX_PRIORITY, lclose},
     {"co", OK_NO_ARG, 1, 1, PREFIX_PRIORITY, lcontinue},
     {"contents", 0, 0, 0, PREFIX_PRIORITY, lcontents},
@@ -113,8 +134,12 @@ PRIMTYPE prims[] = {
     {"cos", 1, 1, 1, PREFIX_PRIORITY, lcos},
     {"count", 1, 1, 1, PREFIX_PRIORITY, lcount},
     {"cs", 0, 0, 0, PREFIX_PRIORITY, lclearscreen},
+    {"cslsload", 1, 1, 1, PREFIX_PRIORITY, lcslsload},
     {"ct", 0, 0, 0, PREFIX_PRIORITY, lcleartext},
     {"cursor", 0, 0, 0, PREFIX_PRIORITY, lcursor},
+#ifdef HAVE_WX
+    {"decreasefont", 0, 0, 0, PREFIX_PRIORITY, DecreaseFont},
+#endif
     {"define", 2, 2, 2, PREFIX_PRIORITY, ldefine},
     {"definedp", 1, 1, 1, PREFIX_PRIORITY, ldefinedp},
     {"defined?", 1, 1, 1, PREFIX_PRIORITY, ldefinedp},
@@ -139,12 +164,21 @@ PRIMTYPE prims[] = {
     {"erpls", 0, 0, 0, PREFIX_PRIORITY, lerpls},
     {"erps", 0, 0, 0, PREFIX_PRIORITY, lerps},
     {"error", 0, 0, 0, PREFIX_PRIORITY, lerror},
+#ifdef OBJECTS
+    {"exist", 0, 0, 0, PREFIX_PRIORITY, lexist},
+#endif
     {"exp", 1, 1, 1, PREFIX_PRIORITY, lexp},
     {"fd", 1, 1, 1, PREFIX_PRIORITY, lforward},
     {"fence", 0, 0, 0, PREFIX_PRIORITY, lfence},
     {"fill", 0, 0, 0, PREFIX_PRIORITY, lfill},
+#ifdef HAVE_WX
+    {"filled", 2, 2, 2, PREFIX_PRIORITY, lfilled},
+#endif
     {"first", 1, 1, 1, PREFIX_PRIORITY, lfirst},
     {"firsts", 1, 1, 1, PREFIX_PRIORITY, lfirsts},
+#ifdef HAVE_WX
+    {"font", 0, 0, 0, PREFIX_PRIORITY, lfont},
+#endif
     {"forever", 1, 1, 1, MACRO_PRIORITY, lforever},
     {"form", 3, 3, 3, PREFIX_PRIORITY, lform},
     {"forward", 1, 1, 1, PREFIX_PRIORITY, lforward},
@@ -158,6 +192,11 @@ PRIMTYPE prims[] = {
     {"gprop", 2, 2, 2, PREFIX_PRIORITY, lgprop},
     {"greaterp", 2, 2, 2, PREFIX_PRIORITY, lgreaterp},
     {"greater?", 2, 2, 2, PREFIX_PRIORITY, lgreaterp},
+    {"greaterequalp", 2, 2, 2, PREFIX_PRIORITY, lgreaterequalp},
+    {"greaterequal?", 2, 2, 2, PREFIX_PRIORITY, lgreaterequalp},
+#ifdef OBJECTS
+    {"have", 1, 1, 1, PREFIX_PRIORITY, lhave},
+#endif
     {"heading", 0, 0, 0, PREFIX_PRIORITY, lheading},
     {"help", OK_NO_ARG, 1, 1, PREFIX_PRIORITY, lhelp},
     {"hideturtle", 0, 0, 0, PREFIX_PRIORITY, lhideturtle},
@@ -169,15 +208,26 @@ PRIMTYPE prims[] = {
     {"iffalse", 1, 1, 1, MACRO_PRIORITY, liffalse},
     {"ift", 1, 1, 1, MACRO_PRIORITY, liftrue},
     {"iftrue", 1, 1, 1, MACRO_PRIORITY, liftrue},
+#ifdef HAVE_WX
+    {"increasefont", 0, 0, 0, PREFIX_PRIORITY, IncreaseFont},
+#endif
     {"int", 1, 1, 1, PREFIX_PRIORITY, linteg},
     {"item", 2, 2, 2, PREFIX_PRIORITY, litem},
     {"keyp", 0, 0, 0, PREFIX_PRIORITY, lkeyp},
     {"key?", 0, 0, 0, PREFIX_PRIORITY, lkeyp},
+#ifdef OBJECTS
+    {"kindof", 1, 1, -1, PREFIX_PRIORITY, lkindof},
+#endif
     {"label", 1, 1, 1, PREFIX_PRIORITY, llabel},
+#ifdef HAVE_WX
+    {"labelsize", 0, 0, 0, PREFIX_PRIORITY, llabelsize},	
+#endif
     {"last", 1, 1, 1, PREFIX_PRIORITY, llast},
     {"left", 1, 1, 1, PREFIX_PRIORITY, lleft},
     {"lessp", 2, 2, 2, PREFIX_PRIORITY, llessp},
     {"less?", 2, 2, 2, PREFIX_PRIORITY, llessp},
+    {"lessequalp", 2, 2, 2, PREFIX_PRIORITY, llessequalp},
+    {"lessequal?", 2, 2, 2, PREFIX_PRIORITY, llessequalp},
     {"list", 0, 2, -1, PREFIX_PRIORITY, llist},
     {"listp", 1, 1, 1, PREFIX_PRIORITY, llistp},
     {"listtoarray", 1, 1, 2, PREFIX_PRIORITY, llisttoarray},
@@ -187,6 +237,9 @@ PRIMTYPE prims[] = {
     {"loadpict", 1, 1, 1, PREFIX_PRIORITY, lloadpict},
     {"local", 1, 1, -1, PREFIX_PRIORITY, llocal},
     {"log10", 1, 1, 1, PREFIX_PRIORITY, llog10},
+#ifdef OBJECTS
+    {"logo", 0, 0, 0, PREFIX_PRIORITY, llogo},
+#endif
     {"lowercase", 1, 1, 1, PREFIX_PRIORITY, llowercase},
     {"lput", 2, 2, 2, PREFIX_PRIORITY, llput},
     {"lshift", 2, 2, 2, PREFIX_PRIORITY, llshift},
@@ -194,12 +247,23 @@ PRIMTYPE prims[] = {
     {"macrop", 1, 1, 1, PREFIX_PRIORITY, lmacrop},
     {"macro?", 1, 1, 1, PREFIX_PRIORITY, lmacrop},
     {"make", 2, 2, 2, PREFIX_PRIORITY, lmake},
+#ifndef HAVE_WX
+#if defined(WIN32)
+    {"maximize.window", 1, 1, 1, PREFIX_PRIORITY, maximize},
+#endif
+#endif
     {"member", 2, 2, 2, PREFIX_PRIORITY, lmember},
     {"memberp", 2, 2, 2, PREFIX_PRIORITY, lmemberp},
     {"member?", 2, 2, 2, PREFIX_PRIORITY, lmemberp},
     {"minus", 1, 1, 1, PREFIX_PRIORITY, lsub},
     {"modulo", 2, 2, 2, PREFIX_PRIORITY, lmodulo},
     {"mousepos", 0, 0, 0, PREFIX_PRIORITY, lmousepos},
+#ifdef OBJECTS
+    {"mynamep", 1, 1, 1, PREFIX_PRIORITY, lmynamep},
+    {"mynames", 0, 0, 0, PREFIX_PRIORITY, lmynames},
+    {"myprocp", 1, 1, 1, PREFIX_PRIORITY, lmyprocp},
+    {"myprocs", 0, 0, 0, PREFIX_PRIORITY, lmyprocs},
+#endif
     {"namep", 1, 1, 1, PREFIX_PRIORITY, lnamep},
     {"name?", 1, 1, 1, PREFIX_PRIORITY, lnamep},
     {"names", 0, 0, 0, PREFIX_PRIORITY, lnames},
@@ -207,8 +271,13 @@ PRIMTYPE prims[] = {
     {"nodribble", 0, 0, 0, PREFIX_PRIORITY, lnodribble},
     {"norefresh", 0, 0, 0, PREFIX_PRIORITY, lnorefresh},
     {"not", 1, 1, 1, PREFIX_PRIORITY, lnot},
+    {"notequalp", 2, 2, 2, PREFIX_PRIORITY, lnotequalp},
+    {"notequal?", 2, 2, 2, PREFIX_PRIORITY, lnotequalp},
     {"numberp", 1, 1, 1, PREFIX_PRIORITY, lnumberp},
     {"number?", 1, 1, 1, PREFIX_PRIORITY, lnumberp},
+#ifdef OBJECTS
+    {"oneof", 1, 1, -1, MACRO_PRIORITY, loneof},
+#endif
     {"op", 1, 1, 1, OUTPUT_PRIORITY, loutput},
     {"openappend", 1, 1, 1, PREFIX_PRIORITY, lopenappend},
     {"openread", 1, 1, 1, PREFIX_PRIORITY, lopenread},
@@ -217,6 +286,9 @@ PRIMTYPE prims[] = {
     {"or", 0, 2, -1, PREFIX_PRIORITY, lor},
     {"output", 1, 1, 1, OUTPUT_PRIORITY, loutput},
     {"palette", 1, 1, 1, PREFIX_PRIORITY, lpalette},
+#ifdef OBJECTS
+    {"parents", 0, 0, 0, PREFIX_PRIORITY, lparents},
+#endif
     {"parse", 1, 1, 1, PREFIX_PRIORITY, lparse},
     {"pause", 0, 0, 0, PREFIX_PRIORITY, lpause},	       
     {"pc", 0, 0, 0, PREFIX_PRIORITY, lpencolor},
@@ -247,7 +319,13 @@ PRIMTYPE prims[] = {
     {"prefix", 0, 0, 0, PREFIX_PRIORITY, lprefix},
     {"primitivep", 1, 1, 1, PREFIX_PRIORITY, lprimitivep},
     {"primitive?", 1, 1, 1, PREFIX_PRIORITY, lprimitivep},
+    {"primitives", 0, 0, 0, PREFIX_PRIORITY, lprimitives},
     {"print", 0, 1, -1, PREFIX_PRIORITY, lprint},
+    {"printout", 1, 1, 1, PREFIX_PRIORITY, lpo},
+#ifdef HAVE_WX
+    {"printpict", 0, 0, 1, PREFIX_PRIORITY, lprintpict},
+    {"printtext", 0, 0, 1, PREFIX_PRIORITY, lprinttext},
+#endif
     {"procedurep", 1, 1, 1, PREFIX_PRIORITY, lprocedurep},
     {"procedure?", 1, 1, 1, PREFIX_PRIORITY, lprocedurep},
     {"procedures", 0, 0, 0, PREFIX_PRIORITY, lprocedures},
@@ -258,7 +336,7 @@ PRIMTYPE prims[] = {
     {"radarctan", 1, 1, 2, PREFIX_PRIORITY, lradatan},
     {"radcos", 1, 1, 1, PREFIX_PRIORITY, lradcos},
     {"radsin", 1, 1, 1, PREFIX_PRIORITY, lradsin},
-    {"random", 1, 1, 1, PREFIX_PRIORITY, lrandom},
+    {"random", 1, 1, 2, PREFIX_PRIORITY, lrandom},
     {"rawascii", 1, 1, 1, PREFIX_PRIORITY, lrawascii},
     {"rc", 0, 0, 0, PREFIX_PRIORITY, lreadchar},
     {"rcs", 1, 1, 1, PREFIX_PRIORITY, lreadchars},
@@ -274,6 +352,9 @@ PRIMTYPE prims[] = {
     {"remprop", 2, 2, 2, PREFIX_PRIORITY, lremprop},
     {"repcount", 0, 0, 0, PREFIX_PRIORITY, lrepcount},
     {"repeat", 2, 2, 2, MACRO_PRIORITY, lrepeat},
+#ifdef OBJECTS
+    {"representation", 0, 0, 0, PREFIX_PRIORITY, lrepresentation},
+#endif
     {"rerandom", 0, 0, 1, PREFIX_PRIORITY, lrerandom},
     {"right", 1, 1, 1, PREFIX_PRIORITY, lright},
     {"rl", 0, 0, 0, PREFIX_PRIORITY, lreadlist},
@@ -283,20 +364,30 @@ PRIMTYPE prims[] = {
     {"runparse", 1, 1, 1, PREFIX_PRIORITY, lrunparse},
     {"runresult", 1, 1, 1, MACRO_PRIORITY, lrunresult},
     {"rw", 0, 0, 0, PREFIX_PRIORITY, lreadword},
-    {"save", 1, 1, 1, PREFIX_PRIORITY, lsave},
+    {"save", OK_NO_ARG, 1, 1, PREFIX_PRIORITY, lsave},
     {"savepict", 1, 1, 1, PREFIX_PRIORITY, lsavepict},
     {"screenmode", 0, 0, 0, PREFIX_PRIORITY, lscreenmode},
     {"scrunch", 0, 0, 0, PREFIX_PRIORITY, lscrunch},
     {"se", 0, 2, -1, PREFIX_PRIORITY, lsentence},
+#ifdef OBJECTS
+    {"self", 0, 0, 0, PREFIX_PRIORITY, lself},
+#endif
     {"sentence", 0, 2, -1, PREFIX_PRIORITY, lsentence},
     {"setbg", 1, 1, 1, PREFIX_PRIORITY, lsetbackground},
     {"setbackground", 1, 1, 1, PREFIX_PRIORITY, lsetbackground},
     {"setcursor", 1, 1, 1, PREFIX_PRIORITY, lsetcursor},
+    {"setcslsloc", 1, 1, 1, PREFIX_PRIORITY, lsetcslsloc},
+#ifdef HAVE_WX
+    {"setfont", 1, 1, 1, PREFIX_PRIORITY, lsetfont},
+#endif
     {"seteditor", 1, 1, 1, PREFIX_PRIORITY, lseteditor},
     {"seth", 1, 1, 1, PREFIX_PRIORITY, lsetheading},
     {"setheading", 1, 1, 1, PREFIX_PRIORITY, lsetheading},
     {"sethelploc", 1, 1, 1, PREFIX_PRIORITY, lsethelploc},
     {"setitem", 3, 3, 3, PREFIX_PRIORITY, lsetitem},
+#ifdef HAVE_WX
+    {"setlabelheight", 1, 1, 1, PREFIX_PRIORITY, lsetlabelheight},
+#endif
     {"setlibloc", 1, 1, 1, PREFIX_PRIORITY, lsetlibloc},
     {"setmargins", 1, 1, 1, PREFIX_PRIORITY, lsetmargins},
     {"setpalette", 2, 2, 2, PREFIX_PRIORITY, lsetpalette},
@@ -309,9 +400,12 @@ PRIMTYPE prims[] = {
     {"setread", 1, 1, 1, PREFIX_PRIORITY, lsetread},
     {"setreadpos", 1, 1, 1, PREFIX_PRIORITY, lsetreadpos},
     {"setscrunch", 2, 2, 2, PREFIX_PRIORITY, lsetscrunch},
-#if defined(WIN32)|defined(ibm)
+#if defined(WIN32)|defined(ibm)|defined(HAVE_WX)
     {"settc", 2, 2, 2, PREFIX_PRIORITY, set_text_color},
     {"settextcolor", 2, 2, 2, PREFIX_PRIORITY, set_text_color},
+#endif
+#ifdef HAVE_WX
+    {"settextsize", 1, 1, 1, PREFIX_PRIORITY, lsettextsize},
 #endif
     {"settemploc", 1, 1, 1, PREFIX_PRIORITY, lsettemploc},
     {"setwrite", 1, 1, 1, PREFIX_PRIORITY, lsetwrite},
@@ -325,6 +419,9 @@ PRIMTYPE prims[] = {
     {"shown?", 0, 0, 0, PREFIX_PRIORITY, lshownp},
     {"showturtle", 0, 0, 0, PREFIX_PRIORITY, lshowturtle},
     {"sin", 1, 1, 1, PREFIX_PRIORITY, lsin},
+#ifdef OBJECTS
+    {"something", 0, 0, 0, PREFIX_PRIORITY, lsomething},
+#endif
     {"splitscreen", 0, 0, 0, PREFIX_PRIORITY, lsplitscreen},
     {"sqrt", 1, 1, 1, PREFIX_PRIORITY, lsqrt},
     {"ss", 0, 0, 0, PREFIX_PRIORITY, lsplitscreen},
@@ -339,9 +436,15 @@ PRIMTYPE prims[] = {
     {"substring?", 2, 2, 2, PREFIX_PRIORITY, lsubstringp},
     {"sum", 0, 2, -1, PREFIX_PRIORITY, ladd},
     {"tag", 1, 1, 1, PREFIX_PRIORITY, ltag},
+#ifdef OBJECTS
+    {"talkto", 1, 1, 1, PREFIX_PRIORITY, ltalkto},
+#endif
     {"test", 1, 1, 1, PREFIX_PRIORITY, ltest},
     {"text", 1, 1, 1, PREFIX_PRIORITY, ltext},
     {"textscreen", 0, 0, 0, PREFIX_PRIORITY, ltextscreen},
+#ifdef HAVE_WX
+    {"textsize", 0, 0, 0, PREFIX_PRIORITY, ltextsize},
+#endif
     {"thing", 1, 1, 1, PREFIX_PRIORITY, lthing},
     {"throw", 1, 1, 2, PREFIX_PRIORITY, lthrow},
     {"to", -1, -1, -1, PREFIX_PRIORITY, lto},
@@ -358,6 +461,8 @@ PRIMTYPE prims[] = {
     {"unstep", 1, 1, 1, PREFIX_PRIORITY, lunstep},
     {"untrace", 1, 1, 1, PREFIX_PRIORITY, luntrace},
     {"uppercase", 1, 1, 1, PREFIX_PRIORITY, luppercase},
+    {"vbarredp", 1, 1, 1, PREFIX_PRIORITY, lvbarredp},
+    {"vbarred?", 1, 1, 1, PREFIX_PRIORITY, lvbarredp},
     {"wait", 1, 1, 1, PREFIX_PRIORITY, lwait},
     {"window", 0, 0, 0, PREFIX_PRIORITY, lwindow},
     {"word", 0, 2, -1, PREFIX_PRIORITY, lword},
@@ -395,7 +500,9 @@ void init(void) {
     int i = 0;
     NODE *iproc = NIL, *pname = NIL, *cnd = NIL;
     FILE *fp;
-    char linebuf[100];
+    char linebuf[256];
+    char *sugar;
+    static char sugarlib[100], sugarhelp[100], sugarcsls[100];
 #ifdef WIN32
     HKEY regKey1, regKey2, regKey3;
     int got_hklm = 0;
@@ -424,8 +531,28 @@ void init(void) {
     i = 0;
 #endif
 
-    logolib = getenv("LOGOLIB");
-    helpfiles = getenv("LOGOHELP");
+    sugar = getenv("SUGAR_BUNDLE_PATH");
+    if (sugar != NULL) {
+	strcpy(sugarlib,sugar);
+	strcat(sugarlib,"/logolib");
+	logolib = sugarlib;
+	fp = fopen(logolib,"r");
+	if (fp == NULL) goto nosugar;
+	fclose(fp);
+	chdir(getenv("SUGAR_ACTIVITY_ROOT"));
+	chdir("data");
+	strcpy(sugarhelp,sugar);
+	strcat(sugarhelp,"/helpfiles");
+	helpfiles = sugarhelp;
+	strcpy(sugarcsls,sugar);
+	strcat(sugarcsls,"/csls");
+	csls = sugarcsls;
+    } else {
+nosugar:
+	logolib = getenv("LOGOLIB");
+	helpfiles = getenv("LOGOHELP");
+	csls = getenv("CSLS");
+    }
     editor = getenv("EDITOR");
 #ifdef WIN32
     if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software", 0,
@@ -444,16 +571,25 @@ void init(void) {
 					    strcpy(logolib, buf);
 		    }
 		}
-	/*	helpfiles = getenv("LOGOLIB"); */
+	/*	helpfiles = getenv("LOGOHELP"); */
 		if (helpfiles == NULL) {
 		    bufsiz=200;
-		    if (RegQueryValueEx(regKey3, "LOGOHELP", NULL,
+		    if (RegQueryValueEx(regKey3, "HELPFILE", NULL,
 					NULL, buf, &bufsiz) == ERROR_SUCCESS) {
 			helpfiles = malloc(bufsiz);
 			strcpy(helpfiles, buf);
 		    }
 		}
-	/*	editor = getenv("LOGOLIB"); */
+	/*	csls = getenv("CSLS"); */
+		if (csls == NULL) {
+		    bufsiz=200;
+		    if (RegQueryValueEx(regKey3, "CSLS", NULL,
+					NULL, buf, &bufsiz) == ERROR_SUCCESS) {
+			csls = malloc(bufsiz);
+			strcpy(csls, buf);
+		    }
+		}
+	/*	editor = getenv("EDITOR"); */
 		if (editor == NULL) {
 		    bufsiz=200;
 		    if (RegQueryValueEx(regKey3, "EDITOR", NULL,
@@ -484,16 +620,25 @@ void init(void) {
 					    strcpy(logolib, buf);
 		    }
 		}
-	/*	helpfiles = getenv("LOGOLIB"); */
+	/*	helpfiles = getenv("LOGOHELP"); */
 		if (helpfiles == NULL) {
 		    bufsiz=200;
-		    if (RegQueryValueEx(regKey3, "LOGOHELP", NULL,
+		    if (RegQueryValueEx(regKey3, "HELPFILE", NULL,
 					NULL, buf, &bufsiz) == ERROR_SUCCESS) {
 			helpfiles = malloc(bufsiz);
 			strcpy(helpfiles, buf);
 		    }
 		}
-	/*	editor = getenv("LOGOLIB"); */
+	/*	csls = getenv("CSLS"); */
+		if (csls == NULL) {
+		    bufsiz=200;
+		    if (RegQueryValueEx(regKey3, "CSLS", NULL,
+					NULL, buf, &bufsiz) == ERROR_SUCCESS) {
+			csls = malloc(bufsiz);
+			strcpy(csls, buf);
+		    }
+		}
+	/*	editor = getenv("EDITOR"); */
 		if (editor == NULL) {
 		    bufsiz=200;
 		    if (RegQueryValueEx(regKey3, "EDITOR", NULL,
@@ -509,9 +654,61 @@ void init(void) {
 	RegCloseKey(regKey1);
     }
 #endif
+
     if (logolib == NULL) logolib = libloc;
     if (helpfiles == NULL) helpfiles = helploc;
+    if (csls == NULL) csls = cslsloc;
+
+#ifdef HAVE_WX
+    //Check for Messages in logolib
+    {
+      FILE *message_fp;
+      char executable_dir[256], message_fn[256], new_logolib[256];
+      snprintf(message_fn, 256,"%s%sMessages", logolib, separator);
+      message_fp = fopen(message_fn, "r");
+      if (message_fp == NULL) {
+        //Messages not in logolib, try relative to executable
+        getExecutableDir(executable_dir, 256);
+        snprintf(new_logolib, 256, "%s%slogolib", executable_dir, separator);
+        snprintf(message_fn, 256, "%s%sMessages", new_logolib, separator);
+        message_fp = fopen(message_fn, "r");
+        if (message_fp != NULL) {
+          printf("Changing logolib from %s to %s\n", logolib, new_logolib);
+          logolib = malloc(sizeof(char)*(strlen(new_logolib)+1));
+          strcpy(logolib, new_logolib);
+        }
+      }
+      if (message_fp != NULL) {
+        fclose(message_fp);
+      }
+    }
+#ifndef __WXMSW__
+	// have to do this because we don't have __WXMAC__ in C
+	const char* wxMacGetCslsloc();
+	const char* wxMacGetHelploc();
+	const char* wxMacGetLibloc();
+	char* newlib;
+	char* newcsls;
+	char* newhelp;
+	// check if we are running wxMac
+	newlib = wxMacGetLibloc();
+	if(newlib)
+		logolib=newlib; 
+    //if (helpfiles == NULL) helpfiles = wxMacGetHelploc();
+    newcsls = wxMacGetCslsloc();
+	if(newcsls)
+		csls = newcsls;
+	newhelp = wxMacGetHelploc();
+    if(newhelp)
+		helpfiles = newhelp;
+#endif
+#endif
+	
+#ifdef unix
+    if (editor == NULL) editor = "emacs";
+#else
     if (editor == NULL) editor = "jove";
+#endif
     editorname = strrchr(editor, (int)'/');
     if (editorname == NULL) editorname = strrchr(editor, (int)'\\');
 #ifdef WIN32
@@ -552,6 +749,8 @@ void init(void) {
 	setprocnode__caseobj(cnd, iproc);
 	if (nodetype(iproc) == MACRO)
 	    setflag__caseobj(cnd, PROC_MACRO);
+	if (prims[i].minargs < 0)
+	    setflag__caseobj(cnd, PROC_SPECFORM);
 	setflag__caseobj(cnd, PERMANENT);
 	i++;
     }
@@ -564,6 +763,8 @@ void init(void) {
     Redefp = intern_p(make_static_strnode("redefp"));
     Caseignoredp = intern_p(make_static_strnode("caseignoredp"));
     Erract = intern_p(make_static_strnode("erract"));
+    Buttonact = intern_p(make_static_strnode("buttonact"));
+    Keyact = intern_p(make_static_strnode("Keyact"));
     Printdepthlimit = intern_p(make_static_strnode("printdepthlimit"));
     Printwidthlimit = intern_p(make_static_strnode("printwidthlimit"));
     LoadNoisily = intern_p(make_static_strnode("loadnoisily"));
@@ -576,15 +777,24 @@ void init(void) {
     Dotsvalue = make_colon(car(Listvalue));
     Pause = intern_p(make_static_strnode("pause"));
     Startup = intern_p(make_static_strnode("startup"));
+    Startuplg = intern_p(make_static_strnode("startup.lg"));
+    LogoVersion = intern_p(make_static_strnode("logoversion"));
+    LogoPlatform = intern_p(make_static_strnode("logoplatform"));
+    LogoLogo = intern_p(make_static_strnode("logo-logo"));
+    CommandLine = intern_p(make_static_strnode("command.line"));
+    setflag__caseobj(CommandLine, VAL_BURIED);
     the_generation = cons(NIL, NIL);
     Not_Enough_Node = cons(NIL, NIL);
 
-    sprintf(linebuf,"%s%sMessages", logolib, separator);
+    snprintf(linebuf, 256, "%s%sMessages", logolib, separator);
     fp = fopen("Messages", "r");
     if (fp == NULL)
 	fp = fopen(linebuf, "r");
+    if (fp == NULL)
+	fp = fopen("C:\\cygwin\\usr\\local\\lib\\logo\\logolib\\Messages", "r");
     if (fp == NULL) {
 	printf("Error -- Can't read Messages file.\n");
+	printf("logolib: %s\n", logolib);
 	exit(1);
     }
 
@@ -620,6 +830,9 @@ void init(void) {
     setvalnode__caseobj(UseAlternateNames, False);
     setflag__caseobj(UseAlternateNames, VAL_BURIED);
 
+    setvalnode__caseobj(LogoPlatform, make_static_strnode(LogoPlatformName));
+    setflag__caseobj(LogoPlatform, VAL_BURIED);
+
     user_repcount = -1;
     ift_iff_flag = -1;
 
@@ -627,6 +840,12 @@ void init(void) {
     Regs_Node = newnode(STACK);
     Regs_Node->n_car = (NODE *)&regs;
 
+//hist_inptr = hist_outptr = cmdHistory;
+
+#ifdef OBJECTS
+    obj_init();
+#endif
+
 /*  Uncomment these to print debugging messages right away! */
 /*
     setvalnode__caseobj(Redefp, True);
