@@ -3,19 +3,18 @@
  *
  *	Copyright (C) 1993 by the Regents of the University of California
  *
- *      This program is free software; you can redistribute it and/or modify
+ *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
+ *      the Free Software Foundation, either version 3 of the License, or
  *      (at your option) any later version.
- *  
+ *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
- *  
+ *
  *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *      along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -60,11 +59,11 @@ NODE *make_case(NODE *casestrnd, NODE *obj) {
     return(new_caseobj);
 }
 
-NODE *make_object(NODE *canonical, NODE *proc, NODE *val,
+NODE *make_object(NODE *canonical, NODE *oproc, NODE *val,
 		  NODE *plist, NODE *casestrnd) {
     NODE *temp;
 
-    temp = cons_list(0, canonical, proc, val, plist,
+    temp = cons_list(0, canonical, oproc, val, plist,
 		     make_intnode((FIXNUM)0), END_OF_LIST);
     make_case(casestrnd, temp);
     return(temp);
