@@ -3,19 +3,18 @@
  *
  *	Copyright (C) 1993 by the Regents of the University of California
  *
- *      This program is free software; you can redistribute it and/or modify
+ *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
+ *      the Free Software Foundation, either version 3 of the License, or
  *      (at your option) any later version.
- *  
+ *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
- *  
+ *
  *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *      along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "logo.h"
@@ -233,7 +232,7 @@ NODE *lrawascii(NODE *args) {
     return(val);
 }
 
-NODE *lbackslashedp(NODE *args) {
+NODE *lvbarredp(NODE *args) {
     char i;
     NODE *arg;
 
@@ -724,6 +723,8 @@ NODE *lform(NODE *args) {
     int width, precision = 0;
     char result[100];
     char format[20];
+    char *old_stringptr = print_stringptr;
+    int old_stringlen = print_stringlen;
 
     number = float_arg(args);
     width = (int)int_arg(cdr(args));
@@ -732,6 +733,8 @@ NODE *lform(NODE *args) {
 	print_stringlen = 20;
 	ndprintf((FILE *)NULL,"%p\n",string_arg(cddr(args)));
 	*print_stringptr = '\0';
+	print_stringptr = old_stringptr;
+	print_stringlen = old_stringlen;
     } else
 	precision = (int)int_arg(cddr(args));
     if (NOT_THROWING) {
