@@ -3,19 +3,18 @@
  *
  *	Copyright (C) 1993 by the Regents of the University of California
  *
- *      This program is free software; you can redistribute it and/or modify
+ *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
+ *      the Free Software Foundation, either version 3 of the License, or
  *      (at your option) any later version.
- *  
+ *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
- *  
+ *
  *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *      along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -23,12 +22,13 @@
 #ifndef _LOGO_H
 #define _LOGO_H
 
-/* #define OBJECTS  */
+// #define OBJECTS 
 
 /* #define MEM_DEBUG */
 
 #define ecma	/* for European extended character set using parity bit */
 
+
 #ifdef WIN32
 #define ibm
 #undef __RZTC__
@@ -88,10 +88,16 @@
 extern char *getenv();
 #endif
 
+#ifndef HAVE_WX
 #ifdef ibm
 #include <dos.h>
 #endif
+#endif
 
+#ifdef HAVE_WX
+#undef ibm
+#define check_throwing (check_wx_stop(0) || stopping_flag == THROWING)
+#else
 #ifdef mac
 #define check_throwing (check_mac_stop() || stopping_flag == THROWING)
 #else
@@ -105,6 +111,7 @@ extern char *getenv();
 #endif
 #endif
 #endif
+#endif
 
 typedef enum {wrapmode, fencemode, windowmode} mode_type;
 
@@ -127,6 +134,7 @@ typedef enum {wrapmode, fencemode, windowmode} mode_type;
 #endif
 #define MAX_PHYS_LINE   5000
 #define MAX_NUMBER      200	/* max number of digits in a float */
+#define HIST_MAX	50	/* number of remembered instruction lines */
 
 #define STOP_PRIORITY	0
 #define OUTPUT_PRIORITY	1
@@ -138,7 +146,8 @@ typedef enum {wrapmode, fencemode, windowmode} mode_type;
 typedef long int NODETYPES;
 
 #ifdef OBJECTS
-#define NT_OBJ    (NODETYPES)040000000
+#define NT_METHOD   (NODETYPES)0100000000
+#define NT_OBJ	    (NODETYPES)040000000
 #endif
 #define NT_LOCAL	(NODETYPES)020000000
 #define NT_STACK	(NODETYPES)010000000
@@ -189,6 +198,7 @@ typedef long int NODETYPES;
 #define LOCALSAVE	(NODETYPES)(CONS|NT_LOCAL)
 #ifdef OBJECTS
 #define OBJECT      (NODETYPES)(NT_OBJ)
+#define METHOD	    (NODETYPES)(NT_AGGR|NT_LIST|NT_METHOD)
 #endif
 
 #define aggregate(nd)	(nodetype(nd) & NT_AGGR)
@@ -204,6 +214,7 @@ typedef long int NODETYPES;
 #define is_tailform(nd)	(nodetype(nd) == TAILFORM)
 #ifdef OBJECTS
 #define is_object(nd)   (nodetype(nd) == NT_OBJ)
+#define is_method(nd)   (nodetype(nd) == NT_METHOD)
 #endif
 
 typedef enum { FATAL, OUT_OF_MEM, STACK_OVERFLOW, TURTLE_OUT_OF_BOUNDS,
@@ -217,9 +228,12 @@ typedef enum { FATAL, OUT_OF_MEM, STACK_OVERFLOW, TURTLE_OUT_OF_BOUNDS,
 		OUT_OF_MEM_UNREC, USER_ERR_MESSAGE, DEEPEND_NONAME,
 		BAD_DEFAULT, RUNRES_STOP, MISSING_SPACE,
 		CANT_OPEN_ERROR, ALREADY_OPEN_ERROR, NOT_OPEN_ERROR,
-#ifdef OBJECTS
+		RUNNABLE_ARG,
+// I removed conditional inclusion of LOCAL_AND_OBJ because
+// it the messages text file does not have the ability to 
+// conditionally include the text for the message, resulting
+// in a shift by 1 when compiled with OBJECTS
 		LOCAL_AND_OBJ,
-#endif
     /* below this point aren't actually error codes, just messages */
 		THANK_YOU, NICE_DAY, NOSHELL_MAC, TYPE_EXIT, ERROR_IN,
 		ERRACT_LOOP, PAUS_ING, TRACE_STOPS, TRACE_OUTPUTS,
@@ -238,6 +252,19 @@ typedef int BOOLEAN;
 #define FALSE	0
 #define TRUE	1
 
+#define SPECIAL_COLORS 5
+#define FILLED_COLOR_INDEX 0
+#define FILLED_COLOR_OFFSET -5
+#define BACKGROUND_COLOR_INDEX 1
+#define BACKGROUND_COLOR_OFFSET -4
+#define PEN_COLOR_INDEX 2
+#define PEN_COLOR_OFFSET -3
+#define TEXT_BG_COLOR_INDEX 3
+#define TEXT_BG_COLOR_OFFSET -2
+#define TEXT_FG_COLOR_INDEX 4
+#define TEXT_FG_COLOR_OFFSET -1
+
+
 #define even_p(x) !(x & 0x1)
 
 #define FIXNUM          long
@@ -293,6 +320,10 @@ typedef struct logo_node {
 	    struct logo_node *nprocs;
 	    struct logo_node *nparents;
 	} nobject;
+	struct {
+	    struct logo_node *nparent;
+	    struct logo_node *nprocname;
+	} nmethod;
 #endif
     } nunion;
 } NODE;
@@ -370,6 +401,11 @@ typedef struct logo_node {
 
 #define getsymbol(frame)          car(frame)
 #define getvalue(frame)           cadr(frame)
+
+#define n_parent		nunion.nmethod.nparent
+#define n_procname		nunion.nmethod.nprocname
+#define getparent(node)		((node)->n_parent)
+#define getprocname(node)	((node)->n_procname)
 #endif	/* OBJECTS */
 
 #ifdef ecma
@@ -478,6 +514,8 @@ struct segment {
 #define clearflag__caseobj(c,f) ((obflags__caseobj(c))->n_int &= ~(f))
 #define flag__caseobj(c,f) (int)((obflags__caseobj(c))->n_int & (f))
 #define flag__object(o,f) (int)((obflags__object(o))->n_int & (f))
+#define setflag__object(c,f) ((obflags__object(c))->n_int |= (f))
+#define clearflag__object(c,f) ((obflags__object(c))->n_int &= ~(f))
 #define is_macro(c) (flag__caseobj(c, PROC_MACRO))
 
 #define push(obj, stack)    stack = cons(obj, stack)
@@ -536,7 +574,9 @@ enum labels {
     x(if) x(ifelse) x(to) x(macro) x(toplevel) x(system) x(error) x(nothing) \
     x(textscreen) x(splitscreen) x(fullscreen) x(paint) x(erase) x(reverse) \
     x(wrap) x(fence) x(window) x(sum) x(difference) x(product) x(quotient) \
-    x(equalp) x(lessp) x(greaterp) x(name) x(class) x(self) \
+    x(equalp) x(lessp) x(greaterp) x(lessequalp) x(greaterequalp) \
+    x(notequalp) \
+    x(name) x(class) x(self) \
     x(licenseplate) x(initlist) x(exist)
 #else
 #define do_trans(x) \
@@ -594,6 +634,11 @@ struct registers {
     FIXNUM r_repcount;	    /* count for repeat */
     FIXNUM r_user_repcount;
     FIXNUM r_ift_iff_flag;
+#ifdef OBJECTS
+    NODE *r_usual_parent;
+    NODE *r_usual_caller;
+#endif
+
 };
 
 /* definitions for evaluator registers */
@@ -620,7 +665,11 @@ struct registers {
 #define user_repcount (regs.r_user_repcount)
 #define ift_iff_flag (regs.r_ift_iff_flag)
 
-#define exp expresn
+#ifdef OBJECTS
+#define usual_parent (regs.r_usual_parent)
+#define usual_caller (regs.r_usual_caller)
+#endif
+
 #endif
 
 #endif /* _LOGO_H */
