@@ -3,19 +3,18 @@
  *
  *	Copyright (C) 1993 by the Regents of the University of California
  *
- *      This program is free software; you can redistribute it and/or modify
+ *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
+ *      the Free Software Foundation, either version 3 of the License, or
  *      (at your option) any later version.
- *  
+ *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
- *  
+ *
  *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *      along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -96,6 +95,18 @@ char *noparity_strnzcpy(char *s1, char *s2, int n) {
     return(s1);
 }
 
+char *backslashed_strnzcpy(char *s1, char *s2, int n) {
+    int i,j;
+
+    for (i = 0, j = 0; i < n; i++) {
+	if (getparity(s2[i]))
+	    s1[j++] = '\\';
+	s1[j++] = clearparity(s2[i]);
+    }
+    s1[j] = '\0';
+    return(s1);
+}
+
 char *mend_strnzcpy(char *s1, char *s2, int n) {
     int i, vbar = 0;
 
@@ -535,6 +546,7 @@ NODE *lpprop(NODE *args) {
 	    setcar(cdr(val), newval);
 	else
 	    setplist__caseobj(plname, cons(pname, cons(newval, plist)));
+	need_save = 1;
     }
     return(UNBOUND);
 }
