@@ -3,19 +3,18 @@
  *
  *	Copyright (C) 1993 by the Regents of the University of California
  *
- *      This program is free software; you can redistribute it and/or modify
+ *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
+ *      the Free Software Foundation, either version 3 of the License, or
  *      (at your option) any later version.
- *  
+ *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
- *  
+ *
  *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *      along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -28,7 +27,11 @@
 #include "globals.h"
 
 #ifdef HAVE_TERMIO_H
+#ifdef HAVE_WX
+#include <termios.h>
+#else
 #include <termio.h>
+#endif
 #else
 #ifdef HAVE_SGTTY_H
 #include <sgtty.h>
@@ -56,6 +59,9 @@ jmp_buf iblk_buf;
 
 NODE *current_line = NIL;
 NODE **bottom_stack; /*GC*/
+NODE *command_line = NIL;   /* 6.0 command line args after files */
+
+int stop_quietly_flag=0;
 
 void unblock_input(void) {
     if (input_blocking) {
@@ -87,7 +93,9 @@ RETSIGTYPE logo_stop()
     } else {
 	charmode_off();
 	to_pending = 0;
-	err_logo(STOP_ERROR,NIL);
+        if (!stop_quietly_flag)
+            err_logo(STOP_ERROR,NIL);
+        stop_quietly_flag = 0;
 #ifdef __RZTC__
 	if (!input_blocking)
 #endif
@@ -136,13 +144,61 @@ RETSIGTYPE mouse_down()
 	if (int_during_gc == 0) int_during_gc = 3;
     } else {
 	line = valnode__caseobj(Buttonact);
-	if (line != UNBOUND)
-	    eval_driver(line);
+	if (line != UNBOUND && line != NIL) {
+	    if (inside_evaluator) {
+		eval_buttonact = line;
+	    } else {
+		eval_driver(line);
+		fix_turtle_shownness();
+	    }
+	}
     }
     SIGRET
 }
 
-RETSIGTYPE (*intfuns[])() = {0, logo_stop, logo_pause, mouse_down};
+int keyact_set() {
+    NODE *line;
+
+    line = valnode__caseobj(Keyact);
+    return (line != UNBOUND && line != NIL);
+}
+
+void do_keyact(int);
+
+#ifdef SIG_TAKES_ARG
+#define sig_arg 0
+RETSIGTYPE delayed_keyact(int sig)
+#else
+#define sig_arg 
+RETSIGTYPE delayed_keyact()
+#endif
+{
+    do_keyact(readchar_lookahead_buf);
+    SIGRET
+}
+
+void do_keyact(int ch) {
+    NODE *line;
+
+    readchar_lookahead_buf = ch;
+    if (inside_gc || in_eval_save) {
+	if (int_during_gc == 0) int_during_gc = 4;
+    } else {
+	line = valnode__caseobj(Keyact);
+	if (line != UNBOUND && line != NIL) {
+	    if (inside_evaluator) {
+		eval_buttonact = line;
+	    } else {
+		eval_driver(line);
+		fix_turtle_shownness();
+	    }
+	}
+    }
+}
+
+
+RETSIGTYPE (*intfuns[])() = {0, logo_stop, logo_pause, mouse_down,
+			     delayed_keyact};
 
 void delayed_int() {
 #ifdef SIG_TAKES_ARG
@@ -159,11 +215,15 @@ void _far _cdecl do_ctrl_c(void) {
 #endif
 
 #ifdef HAVE_WX
+int wx_leave_mainloop = 0;
 int  start (int argc,char ** argv) {
 #else
 int main(int argc, char *argv[]) {
 #endif
     NODE *exec_list = NIL;
+    NODE *cl_tail = NIL;
+    int argc2;
+    char **argv2;
 
 #ifdef SYMANTEC_C
     extern void (*openproc)(void);
@@ -200,29 +260,70 @@ int main(int argc, char *argv[]) {
 #ifdef mac
     signal(SIGQUIT, SIG_IGN);
 #else /* !mac */
-    signal(SIGQUIT, logo_pause);
+	//signal(SIGQUIT, logo_pause);
 #endif
     /* SIGQUITs never happen on the IBM */
 
     if (argc < 2) {
 #ifndef WIN32
-      if (isatty(1))
+      if (1 || isatty(1))   // fix this.  for interactive from menu bar.
 #endif
       {
+#ifdef HAVE_WX
+	extern char *GIT;
+#endif
+	char version[64];
 	lcleartext(NIL);
-	ndprintf(stdout, message_texts[WELCOME_TO], "5.5");
+#ifdef HAVE_WX
+	strcpy(version,"6.1");
+	strcat(version,GIT);
+#else
+	strcpy(version,"5.6");
+#endif
+	ndprintf(stdout, message_texts[WELCOME_TO], version);
 	new_line(stdout);
       }
     }
 
-    setvalnode__caseobj(LogoVersion, make_floatnode(5.5));
+#ifdef HAVE_WX
+    setvalnode__caseobj(LogoVersion, make_floatnode(6.1));
+#else
+    setvalnode__caseobj(LogoVersion, make_floatnode(5.6));
+#endif
     setflag__caseobj(LogoVersion, VAL_BURIED);
 
+    argv2 = argv; argc2 = argc;
+
+    if (!strcmp(*argv+strlen(*argv)-4, "logo")) {
+	argv++;
+	while (--argc > 0 && strcmp(*argv, "-") && NOT_THROWING) {
+	    argv++;
+	}
+    }
 
-    silent_load(Startup, NULL); /* load startup.lg */
     argv++;
-    while (--argc > 0 && NOT_THROWING) {
-	silent_load(NIL,*argv++);
+    while (--argc > 0) {
+	if (command_line == NIL) 
+	    cl_tail = command_line = cons(make_static_strnode(*argv++), NIL);
+	else {
+	    setcdr(cl_tail, cons(make_static_strnode(*argv++), NIL));
+	    cl_tail = cdr(cl_tail);
+	}
+    }
+
+    setvalnode__caseobj(CommandLine, command_line);
+
+    silent_load(Startuplg, logolib);
+#ifndef WIN32
+    silent_load(Startuplg, getenv("HOME")); /* load startup.lg */
+#else
+    silent_load(Startuplg, NULL); /* load startup.lg */
+#endif
+    if (!strcmp(*argv2+strlen(*argv2)-4, "logo")) {
+	argv2++;
+	while (--argc2 > 0 && strcmp(*argv2, "-") && NOT_THROWING) {
+	    silent_load(NIL,*argv2++);
+	    }
     }
 
     for (;;) {
@@ -248,6 +349,11 @@ int main(int argc, char *argv[]) {
 		if (exec_list != NIL) eval_driver(exec_list);
 	    }
 	}
+#ifdef HAVE_WX
+	if (wx_leave_mainloop) {
+	  break;
+	}
+#endif	
 	if (stopping_flag == THROWING) {
 	    if (isName(throw_node, Name_error)) {
 		err_print(NULL);
@@ -264,6 +370,7 @@ int main(int argc, char *argv[]) {
 	    stopping_flag = RUN;
 	}
     }
-    prepare_to_exit(TRUE);
+    //prepare_to_exit(TRUE);
+    exit(0);
     return 0;
 }
