@@ -3,25 +3,24 @@
  *
  *	Copyright (C) 1993 by the Regents of the University of California
  *
- *      This program is free software; you can redistribute it and/or modify
+ *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
+ *      the Free Software Foundation, either version 3 of the License, or
  *      (at your option) any later version.
- *  
+ *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
- *  
+ *
  *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *      along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #define WANT_EVAL_REGS 1
 #include "logo.h"
 #include "globals.h"
-extern NODE *stack, *numstack, *exp, *val, *parm, *catch_tag, *arg;
+extern NODE *stack, *numstack, *expresn, *val, *parm, *catch_tag, *arg;
 
 /* #ifdef ibm */
 /* #ifndef __RZTC__ */
@@ -102,6 +101,7 @@ int next_gen_gc = 0, max_gen = 0;
 int mark_gen_gc;
 
 /* #define GC_DEBUG 1 /* */
+/* #define GC_TWOBYTE 1 /* Use 2-byte stack offset in mark phase */
 
 #ifdef GC_DEBUG
 long int num_examined;
@@ -289,6 +289,10 @@ NODE **inter_gen_mark (NODE **prev) {
 	case TREE:
 	case LINE:
 	case LOCALSAVE:
+#ifdef OBJECTS
+	case OBJECT:
+	case METHOD:
+#endif
 	    if (valid_pointer(nd->n_car))
 		mmark(nd->n_car);
 	    if (valid_pointer(nd->n_obj))
@@ -318,11 +322,13 @@ NODE **inter_gen_mark (NODE **prev) {
 	    }
 	    break;
     }
+#ifdef WHYDOESNTTHISWORK
     if (!got_young) {	/* nd no longer points to younger */
 	*prev = nd->oldyoung_next;
    	nd->oldyoung_next = NIL;
    	return prev;
     }
+#endif
     return &(nd->oldyoung_next);
 }
 
@@ -409,6 +415,10 @@ void mark(NODE* nd) {
 		case TREE:
 		case LINE:
 		case LOCALSAVE:
+#ifdef OBJECTS
+		case OBJECT:
+		case METHOD:
+#endif
 		    *gctop = nd->n_car;
 		    gc_inc();
 		    *gctop = nd->n_obj;
@@ -508,7 +518,7 @@ re_mark:
     mark(Regs_Node);
     mark(stack);
     mark(numstack);
-    mark(exp);
+    mark(expresn);
     mark(val);
     mark(parm);
     mark(catch_tag);
@@ -538,6 +548,7 @@ re_mark:
     mark(reader_name);
     mark(writer_name);
     mark(file_prefix);
+    mark(save_name);
 
     mark(the_generation);
     mark(Not_Enough_Node);
@@ -551,6 +562,12 @@ re_mark:
 
     mark(deepend_proc_name);
 
+#ifdef OBJECTS
+	mark(logo_object);
+	mark(current_object);
+	mark(askexist);
+#endif
+
 #ifdef GC_DEBUG
     printf("globals %ld + ", num_examined);
     if (dribblestream != NULL) fprintf(dribblestream,"globals %ld + ", num_examined);
@@ -570,7 +587,7 @@ re_mark:
 
     if (top_stack < bottom_stack) { /* check direction stack grows */
 	for (tmp_ptr = top_stack; tmp_ptr <= bottom_stack; 
-#if defined(THINK_C) || defined(__RZTC__)
+#if defined(THINK_C) || defined(__RZTC__) || defined(GC_TWOBYTE)
 	     tmp_ptr = (NODE **)(((unsigned long int)tmp_ptr)+2)
 #else
 	     tmp_ptr++
@@ -582,7 +599,7 @@ re_mark:
 	}
     } else {
 	for (tmp_ptr = top_stack; tmp_ptr >= bottom_stack; 
-#if defined(THINK_C) || defined(__RZTC__)
+#if defined(THINK_C) || defined(__RZTC__) || defined(GC_TWOBYTE)
 	     tmp_ptr = (NODE **)(((unsigned long int)tmp_ptr)-2)
 #else
 	     tmp_ptr--
@@ -683,6 +700,10 @@ re_mark:
 			case TREE:
 			case LINE:
 			case LOCALSAVE:
+#ifdef OBJECTS
+			case OBJECT:
+			case METHOD:
+#endif
 			    check_oldyoung(nd, nd->n_car);
 			    check_oldyoung(nd, nd->n_obj);
 			case CONT:
