@@ -3,22 +3,28 @@
  *
  *	Copyright (C) 1993 by the Regents of the University of California
  *
- *      This program is free software; you can redistribute it and/or modify
+ *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
+ *      the Free Software Foundation, either version 3 of the License, or
  *      (at your option) any later version.
- *  
+ *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
- *  
+ *
  *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *      along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
+#ifdef HAVE_WX
+#ifdef WIN32
+#define NEW_WIN32
+#endif
+#undef WIN32
+#endif
+
 #ifdef WIN32
 #include <windows.h>
 #endif
@@ -27,7 +33,13 @@
 #include "globals.h"
 
 #ifdef HAVE_TERMIO_H
+#ifdef HAVE_WX
+#ifndef NEW_WIN32
+#include <termios.h>
+#endif
+#else
 #include <termio.h>
+#endif
 #else
 #ifdef HAVE_SGTTY_H
 #include <sgtty.h>
@@ -47,8 +59,14 @@ extern int getch(void);
 #endif
 
 # ifdef HAVE_WX
+#ifdef getc
+#undef getc
+#endif
 #define getc getFromWX_2
 #define getch getFromWX
+extern int check_wx_stop(int force_yield);
+extern void wx_enable_scrolling();
+extern void wx_refresh();
 #endif
 
 FILE *readstream;
@@ -58,6 +76,12 @@ FILE *dribblestream = NULL;
 int input_blocking = 0;
 NODE *deepend_proc_name = NIL;
 
+#if 0
+char *cmdHistory[HIST_MAX] = {0};
+char **hist_inptr, **hist_outptr;
+#endif
+int readingInstruction = 0;
+
 int rd_getc(FILE *strm) {
     int c;
 #ifdef WIN32
@@ -72,6 +96,9 @@ int rd_getc(FILE *strm) {
     c = getc(strm);
 #endif
     if (strm == stdin && c != EOF) update_coords(c);
+#ifndef mac
+    if (c == '\r') return rd_getc(strm);
+#endif
 #ifdef ibm
     if (c == 17 && interactive && strm==stdin) { /* control-q */
 	to_pending = 0;
@@ -152,6 +179,11 @@ void rd_print_prompt(char *str) {
 #endif
 	lsplitscreen(NIL);
 #endif
+	
+#ifdef HAVE_WX
+	if(in_graphics_mode && !in_splitscreen)
+		lsplitscreen(NIL);
+#endif
 
 #ifdef mac
     extern int in_fscreen(void);
@@ -186,13 +218,34 @@ void zrd_print_prompt(char *str) {
 char *p_line = 0, *p_end;
 int p_len = MAX_PHYS_LINE;
 
+char ender[100];
+
 NODE *reader(FILE *strm, char *prompt) {
     int c = 0, dribbling, vbar = 0, paren = 0;
     int bracket = 0, brace = 0, p_pos, contin=1, insemi=0, raw=0;
-    static char ender[] = "\nEND\n";
     char *phys_line, *lookfor = ender;
     NODETYPES this_type = STRING;
     NODE *ret;
+    char *old_stringptr = print_stringptr;
+    int old_stringlen = print_stringlen;
+
+    fix_turtle_shownness();
+
+    //readingInstruction = !strcmp(prompt, "? ");
+    readingInstruction = (strm == stdin);
+#ifdef HAVE_WX
+    wx_refresh();
+    if(readingInstruction) {
+      wx_enable_scrolling();
+    }
+#endif
+
+    print_stringptr = ender;
+    print_stringlen = 99;
+    ndprintf(NULL, "\n%p\n", theName(Name_end));
+    *print_stringptr = '\0';
+    print_stringptr = old_stringptr;
+    print_stringlen = old_stringlen;
 
     if (!strcmp(prompt, "RW")) {	/* called by readword */
 	    prompt = "";
@@ -235,10 +288,10 @@ charmode_off();
     if (!setjmp(iblk_buf)) {
 #endif
     c = rd_getc(strm);
-    if ((c=='\b' || c=='\127') && strm==stdin && interactive) {
+/*    if ((c=='\b' || c=='\127') && strm==stdin && interactive) {
 	silent_load(LogoLogo, logolib);
 	c = rd_getc(strm);
-    }
+    } */   /* 6.0 */
 #ifdef mac
     if (c == '\r') c = '\n';	/* seen in raw mode by keyp, never read */
 #endif
@@ -258,7 +311,7 @@ charmode_off();
 	    c = rd_getc(strm);
 	    continue;
 	}
-	if (*prompt && (c&0137) == *lookfor) {
+	if (*prompt && (c&0137) == ((*lookfor)&0137)) {
 		lookfor++;
 		if (*lookfor == 0) {
 		    if (deepend_proc_name != NIL)
@@ -268,7 +321,7 @@ charmode_off();
 		    break;
 		}
 	} else lookfor = ender;
-	if (c == '|') {
+	if (c == '|' && !insemi) {
 	    vbar = !vbar;
 	    this_type = VBAR_STRING;
 	} else if (contin && !vbar && !insemi) {
@@ -290,14 +343,25 @@ charmode_off();
 	    }
 	}
 	while (!vbar && c == '~' && (c = rd_getc(strm)) != EOF) {
-	    while (c == ' ' || c == '\t')
+	    int gotspc = 0;
+	    while (c == ' ' || c == '\t') {
+		gotspc = 1;
 		c = rd_getc(strm);
+	    }
 	    if (dribbling) rd_putc(c, dribblestream);
+	    if (c != '\n' && gotspc) into_line(' ');
 	    into_line(c);
 	    if (c == '\n') {
 		insemi = 0;
 		if (interactive && strm == stdin) zrd_print_prompt("~ ");
 	    }
+	    else if (c == '(') paren++;
+	    else if (paren && c == ')') paren--;
+	    else if (c == '[') bracket++;
+	    else if (bracket && c == ']') bracket--;
+	    else if (c == '{') brace++;
+	    else if (brace && c == '}') brace--;
+	    else if (c == ';') insemi++;
 	}
 	if (c != EOF) c = rd_getc(strm);
     }
@@ -319,6 +383,24 @@ charmode_off();
     if (phys_line == p_line) return(Null_Word); /* so emptyp works */
     ret = make_strnode(p_line, (struct string_block *)NULL, (int)strlen(p_line),
 		       this_type, strnzcpy);
+#if 0
+    if (strm == stdin && !strcmp(prompt, "? ")){
+	char *histline = malloc(1+strlen(p_line));
+	strcpy(histline, p_line);
+	*hist_inptr++ = histline;
+	if (hist_inptr >= &cmdHistory[HIST_MAX]) {
+	    hist_inptr = cmdHistory;
+	}
+	if (*hist_inptr) {
+	    free(*hist_inptr);
+	    *hist_inptr = 0;
+	}
+	hist_outptr = hist_inptr;
+    }
+#endif
+
+    //added (evan)
+    readingInstruction = 0;
     return(ret);
 }
 
