@@ -3,19 +3,18 @@
  *
  *	Copyright (C) 1993 by the Regents of the University of California
  *
- *      This program is free software; you can redistribute it and/or modify
+ *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
+ *      the Free Software Foundation, either version 3 of the License, or
  *      (at your option) any later version.
- *  
+ *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
- *  
+ *
  *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *      along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "logo.h"
@@ -40,6 +39,42 @@ void update_coords(char ch) {
     check_scroll();
 #endif
 #endif
+	
+#ifdef HAVE_WX
+	int getTermInfo(int type);
+#if 0
+	int setTermInfo(int type, int val);
+	int x_coord, y_coord, x_max, y_max;
+	x_coord=getTermInfo(X_COORD);
+	y_coord=getTermInfo(Y_COORD);
+	x_max=getTermInfo(X_MAX);
+	y_max=getTermInfo(Y_MAX);
+	
+	if (ch == '\n') {
+		x_coord = 0;
+		y_coord++;
+		setTermInfo(X_COORD,x_coord);
+		setTermInfo(Y_COORD,y_coord);
+		for (i = 0; i < x_margin; i++) print_char(stdout,' ');
+		x_coord=getTermInfo(X_COORD);
+		y_coord=getTermInfo(Y_COORD);
+    } else if (ch == '\b') {
+		if (x_coord > 0) --x_coord;
+    } else if (ch == '\t') {
+		x_coord &= ~07;
+		x_coord += 8;
+    } else if (ch != '\007')
+		x_coord++;
+    if (x_coord > x_max && x_max > 0) {
+		y_coord += x_coord/x_max;
+		x_coord %= x_max;
+    }
+    if (y_coord > y_max) y_coord = y_max;
+	setTermInfo(X_COORD,x_coord);
+	setTermInfo(Y_COORD,y_coord);
+#endif
+	
+#else 	
     if (ch == '\n') {
 	x_coord = 0;
 	y_coord++;
@@ -59,6 +94,7 @@ void update_coords(char ch) {
 	x_coord %= x_max;
     }
     if (y_coord > y_max) y_coord = y_max;
+#endif
 }
 
 void print_char(FILE *strm, char ch) {
@@ -116,7 +152,8 @@ void ndprintf(FILE *strm, char *fmt, ...) {
     va_list ap;
     NODE *nd, *ahead_node = UNBOUND, *next_ahead_node = UNBOUND;
     char *cp;
-    char ch;
+    long int i;
+    char buf[30],ch;
 
     va_start(ap,fmt);
     while ((ch = *fmt++) != '\0') {
@@ -146,6 +183,11 @@ void ndprintf(FILE *strm, char *fmt, ...) {
 	    } else if (ch == 't') { /* text */
 		cp = va_arg(ap,char *);
 		while ((ch = *cp++) != '\0') print_char(strm,ch);
+	    } else if (ch == 'd') { /* integer */
+		i = va_arg(ap, int);
+		sprintf(buf,"%ld",i);
+		cp = buf;
+		while ((ch = *cp++) != '\0') print_char(strm,ch);
 	    } else {
 		print_char(strm,'%');
 		print_char(strm,ch);
@@ -167,7 +209,7 @@ void ndprintf(FILE *strm, char *fmt, ...) {
 }
 
 void dbprint(NODE *data) {
-    ndprintf(stdout, "%p\n", data);
+    ndprintf(stderr, "%p\n", data);
 }
 
 void real_print_help(FILE *strm, NODE *ndlist, int depth, int width) {
